/* ccor.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/stat.h>
#include "tdmdef.h"
#define MAIN
#include "cparam.h"

void error();
void newext();
void swab();

static char fn1[40] = "tdm.tsd";
static char fn2[40] = "tdm.tsr";
static char msg[80] = "";
static int dbgflg = 1;
static int swap_bytes = 0;

/* swabw - swap bytes (in place) for n "words" */
void
swabw(p, n)
long   *p;
int n;
{
    unsigned char *ch, tmp;

    while (n-- > 0) {
	ch = (unsigned char *) p++;
	tmp = ch[3];
	ch[3] = ch[0];
	ch[0] = tmp;
	tmp = ch[2];
	ch[2] = ch[1];
	ch[1] = tmp;
    }
}

void
newext(f, e)
char   *f, *e;
{
    char   *s;

    s = f;
    while (*s != '\0')		/* locate end of filename */
	s++;
    while (*s != '\\' && *s != '/' && s > f)	/* backup to start of name */
	s--;
    while (*s != '\0' && *s != '.')	/* locate end of name */
	s++;
    while (*e != '\0')		/* copy extension */
	*s++ = *e++;
    *s = '\0';
}

void
error(s)
char   *s;
{
    fprintf(stderr, "ERROR: %s\n", s);
    exit(1);
}

void
rd_tsd_hdr(FILE *fp1)
{
    if (fread((char *) &pa_, sizeof(pa_), 1, fp1) < 1) {
	error("reading header\n");
    }
    if (pa_.ireor > 255) {	/* check for byte swap */
	swabw((long *)&pa_.magic, 1);
	swab((void*)&pa_.hdrsiz, (void *)&pa_.hdrsiz, 6 * sizeof(short) );
	swabw((long *)&pa_.dt, 14);
	swab((void*)&pa_.m, (void*)&pa_.m, 21 * sizeof(short));
	swabw((long *)&pa_.xl, 164);
	swap_bytes++;
    }
}

void
rd_tsd_dat(FILE *fp1, float *a, int nrows, int ncols, int swap_bytes)
{
    fseek(fp1, (long)pa_.hdrsiz, 0);
    if (fread((char *) a, sizeof(float) * ncols, nrows, fp1) < nrows) {
        error("reading data\n");
    }
    if (swap_bytes)
        swabw((long *) a, nrows * ncols);
}

static int
wr_mat_hdr(int fd, int m, int p, char *nam, int nr, int nc)
{
    int     len;
    long    hdr[5];

    len = strlen(nam) + 1;
/*
 *  matlab header -
 *	type = MOPT where:
 *	M=0 for pc; 1 for sun
 *	O=0 col or 1 row;
 *	P= [0,1,2,3,4]=>[r*8,r*4,I*4,I*2,I*4 unsigned integers]
 *	T= 0 for matrix or 1 for text (stored as r*4 numbers 0<i<255).
 */
    hdr[0] = m * 1000 + p * 10; /* MOPT */
    hdr[1] = nr;		/* rows */
    hdr[2] = nc;		/* cols */
    hdr[3] = 0;			/* 0 for real, 1 for complex */
    hdr[4] = (long) len;	/* name length (including null byte) */
    (void) write(fd, (char *) hdr, 20);
    (void) write(fd, (char *) nam, len);

    return (20 + len);
}

void
wr_mat(char *fn2, float *b, int ncols)
{
    int     fd, i;

    fd = open(fn2, _O_WRONLY | _O_CREAT | _O_BINARY | _O_TRUNC, S_IREAD | S_IWRITE);
    if (fd < 0)
	return;
    (void) wr_mat_hdr(fd, 0, 1, "cgm", ncols, ncols);
    for (i = 0; i < ncols; i++)
        (void) write(fd, (char *) &b[i * ncols], sizeof(float) * ncols);
    close(fd);
}

void
wr_cor(char *fn2, float *b, int ncols)
{
    int     i;
    FILE   *fp2;

    pa_.ireor = 0;	/* identify output file as reordered data */
    pa_.ntsf = ncols;
    pa_.n = ncols;
    if ((fp2 = fopen(fn2, "wb")) == NULL) {
        sprintf(msg, "can't open %s\n", fn2);
        error(msg);
    }
    if (fwrite((char *) &pa_, sizeof(pa_), 1, fp2) < 1) {
        error("writing header\n");
    }
    fseek(fp2, (long)pa_.hdrsiz, 0);
    for (i = 0; i < ncols; i++)
        if (fwrite((char *) &b[i * ncols], sizeof(float), ncols, fp2) < ncols)
            error("writing data\n");
    fclose(fp2);
}

void
main(int ac, char **av)
{
    double  sum;
    float  *a, *b;
    int     nrows, ncols, npts, i, j, k, matflg = 0;
    FILE   *fp1;

    while (ac > 1 && av[1][0] == '-') {
        if (av[1][1] == 'm') {
            matflg = 1;
        }
        ac--;
        av++;
    }
    if (ac > 2) {
	strcpy(fn1, av[1]);
	newext(fn1, ".tsd");
	strcpy(fn2, av[2]);
	newext(fn2, ".cor");
    } else if (ac > 1) {
	strcpy(fn1, av[1]);
	newext(fn1, ".tsd");
	strcpy(fn2, av[1]);
	newext(fn2, ".cor");
    }
    if (matflg)
        newext(fn2, ".cgm");

    if ((fp1 = fopen(fn1, "rb")) == NULL) {
	sprintf(msg, "can't open %s\n", fn1);
	error(msg);
    }
    fprintf(stderr, "input from %s\n", fn1);
    rd_tsd_hdr(fp1);
    if (dbgflg) {
	printf(" ireor,ifrmt,jfrmt    = %4d%4d%4d\n",
	    pa_.ireor, pa_.ifrmt, pa_.jfrmt);
	printf(" itsc,icyc,itsw,itsf  = %8ld%8ld%8ld%8ld\n",
	    pa_.itsc, pa_.icyc, pa_.itsw, pa_.itsf);
	printf(" ntsc,ncyc,ntsw,ntsf  = %8ld%8ld%8ld%8ld\n",
	    pa_.ntsc, pa_.ncyc, pa_.ntsw, pa_.ntsf);
    }
    if (pa_.ireor != 0 || pa_.ifrmt != 1 || pa_.jfrmt != 6) {
	sprintf(msg, "Wrong type of data file. %s\n",
	    " Program requires ireor,ifrmt,jfrmt =  0  1  6");
	error(msg);
    }
    nrows = pa_.ntsf;
    ncols = pa_.n;

    npts = nrows * ncols;
    printf("nrows=%d  ncols=%d  npts=%d\n", nrows, ncols, npts);

    a = (float *) malloc(nrows * ncols * sizeof(float));
    b = (float *) malloc(nrows * ncols * sizeof(float));
    if (a == NULL || b == NULL) {
	error("allocating memory\n");
    }
    rd_tsd_dat(fp1, a, nrows, ncols, swap_bytes);
    for (i = 0; i < nrows; i++) {
        for (j = 0; j < ncols; j++) {
            if (i == 0 || j == 0 || i >= ncols) {
                b[j + i * ncols] = 0;
            } else {
                sum = 0;
                for (k = 0; k < nrows; k++)
                    sum += a[i + k * ncols] *  a[j + k * ncols];
                b[j + i * ncols] = (sum > 0) ? sqrt(sum / nrows) : 0;
            }
//            b[j + i * ncols] = a[j + i * ncols];
        }
    }
    fclose(fp1);
    free(a);

    fprintf(stderr, "output to %s\n", fn2);
    if (matflg) {
        wr_mat(fn2, b, ncols);
    } else {
        wr_cor(fn2, b, ncols);
    }

    free(b);
    exit(0);
}
