/* datman.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <ctype.h>
#define MAIN
#include "tdmdef.h"
#include "cparam.h"

#define MXAC 16
#define MXNP 2050
#define iscmd(s)   (strcmp(cmd, s) == 0)

int defdev();
int opnout();
void clsout();
void draw();
void move();
void rfft();
void rtop();
void swab();
void text();
void uwph();

static double scalar(char *);
static float *assign(char *);
static int parse_line(char *, char **);
static void do_cmd(FILE *);

static char *lav[MXAC] = {NULL};
static char msg[80];
static double eps = 1e-38;
static double tpi = 6.2831854;
static float aa[MXNP] = {0.0};
static float bb[MXNP] = {0.0};
static float cc[MXNP] = {0.0};
static float dd[MXNP] = {0.0};
static float dtms = 0.0;
static float fsep = 0.0;
static float phrf_rp = 1.5;
static float phrf_uw = 0;
static float tsep = 0.0;
static float xmax = 10.0;
static float xmin = 0.0;
static float ymax = 10.0;
static float ymin = 0.0;
static float zz[MXNP] = {0.0};
static int dbgflg = 0;
static int dspflg = 0;
static int dtset = 0;
static int filopn = 0;
static int lac = 0;
static int npnt, nstp;
static int pdset = 0;
static int swap_bytes = 0;
static int verbose = 0;
static int vpset = 0;

static FILE *dfp = NULL;

static void
error(s)
char *s;
{
    fprintf(stderr, "ERROR:  %s\n", s);
    exit(1);
}

static void
strlower(s)
char *s;
{
    while(*s) {
	if (*s >= 'A' && *s <= 'Z')
		*s += 'a' - 'A';
	s++;
    }
}

static float *
assign(char *s)
{
    if (*s == 'A' || *s == 'a')
	return (aa);
    if (*s == 'B' || *s == 'b')
	return (bb);
    if (*s == 'C' || *s == 'c')
	return (cc);
    if (*s == 'D' || *s == 'd')
	return (dd);
    sprintf(msg, "improper array argument '%s'", s);
    error(msg);
    return (NULL);
}

static void
vderiv(a, b, n)
float *a, *b;
{
    int i;
    float atmp, btmp;

    atmp = a[0];
    for (i = 1; i < n; i++) {
	btmp = a[i];
	b[i] = btmp - atmp;
	atmp = btmp;
    }
}

static void
vinteg(a, b, n)
float *a, *b;
{
    int i;

    for (i = 1; i < n; i++) {
	b[i] = a[i] + a[i - 1];
    }
}

static void
newext(f, e)
char *f, *e;
{
    char *s;
    
    s = f;
    while(*s != '\0')                           /* locate end of filename */
        s++;
    while(*s != '\\' && *s != '/' && s > f)     /* backup to start of name */
        s--;
    while(*s != '\0' && *s != '.')              /* locate end of name */
        s++;
    while(*e != '\0')                           /* copy extension */
        *s++ = *e++;
    *s = '\0';
}

static double
scalar(char *s)
{
    if (*s == 'Q' || *s == 'q')
	return (1);
    if (*s == 'T' || *s == 't')
	return ((pa_.dt * pa_.ntsw) * pa_.ntsf);
    return (atof(s));
}

static void
init_stat()
{
    int i;

    dtms = 0.0;
    fsep = 0.0;
    phrf_rp = 1.5;
    phrf_uw = 0;
    tsep = 0.0;
    xmax = 10.0;
    xmin = 0.0;
    ymax = 10.0;
    ymin = 0.0;
    dtset = 0;
    filopn = 0;
    lac = 0;
    pdset = 0;
    swap_bytes = 0;
    vpset = 0;

    for (i = 0; i < MXNP; i++) {
        aa[i] = 0;
        bb[i] = 0;
        cc[i] = 0;
        dd[i] = 0;
        zz[i] = 0;
    }

}

static int
parse_line(char *s, char **lav)
{
    int lac;

    lac = 0;
    while(*s != '\0') {
	while(*s && *s <= ' ')		/* skip white space */
	    s++;
	if (*s == '#' || *s == '!')	/* strip comments */
	    break;
	if (*s > ' ' && lac < MXAC) {	/* save arg */
	    lav[lac++] = s;
	    while(*s > ' ')
		s++;
	    }
	if (*s == '\0')			/* check for EOL */
	    break;
	*s++ = '\0';			/* terminate arg */
    }
    return (lac);
}

/* swabw - swap bytes (in place) for n "words" */
void
swabw(p, n)
long   *p;
int n;
{
    unsigned char *ch, tmp;

    while (n-- > 0) {
	ch = (unsigned char *) p++;
	tmp = ch[3];
	ch[3] = ch[0];
	ch[0] = tmp;
	tmp = ch[2];
	ch[2] = ch[1];
	ch[1] = tmp;
    }
}

static void
opndat(fn)
char *fn;
{
    if (filopn)
	fclose(dfp);
    if ((dfp = fopen(fn, "rb")) == NULL) {
	sprintf(msg, "can't open %s\n", fn);
	error(msg);
    }
    if (fread((char *) &pa_, sizeof(pa_), 1, dfp) < 1) {
	error("reading header\n");
    }
    if (pa_.ireor > 255) {	/* check for byte swap */
	swabw((long *)&pa_.magic, 1);
	swab((void*)&pa_.hdrsiz, (void *)&pa_.hdrsiz, 6 * sizeof(short) );
	swabw((long *)&pa_.dt, 14);
	swab((void*)&pa_.m, (void*)&pa_.m, 21 * sizeof(short));
	swabw((long *)&pa_.xl, 164);
	swap_bytes++;
    }
    if (pa_.ireor == 2 && pa_.ifrmt == 1) {
	nstp = pa_.ntsf;
        npnt = pa_.n;
    } else if (pa_.ifrmt != 8 && pa_.ifrmt != 7) {
	error("undefined file format");
    } else if (pa_.ireor == 1) {
	nstp = 16;
        npnt = pa_.ntsf;
    } else if (pa_.ireor == 2) {
	nstp = 1;
        npnt = pa_.ntsf;
    } else {
	error("undefined file type");
    }
    dtms = 1000 * pa_.ntsw * pa_.dt;
    if (verbose)
        printf("opened tdm file: %s, npnt=%d, nstp=%d\n", fn, npnt, nstp);
    if (npnt > MXNP) {
	npnt = MXNP;
	printf("WARNING: reducing npnt to %d\n", npnt);
    }
    filopn = 1;
}

static void
do_as()
{
    if (lac < 2)
	error("'AS' needs a filename");
    opndat(lav[1]);
}

static void
do_cm()
{
    int i, j;
    float *ff, *gg, *hh;

    ff = assign(lav[1]);
    gg = assign(lav[2]);
    hh = assign(lav[3]);
    for (i = 0; i < npnt; i += 2) {
	j = i + 1;
	hh[i] = ff[i] * gg[i] - ff[j] * gg[j];
	hh[j] = ff[i] * gg[j] + ff[j] * gg[i];
    }
}

static void
do_de()
{
    int i, n;
    float rdf, ap, am, de;
    float *ff, *gg;

    ff = assign(lav[1]);
    gg = assign(lav[2]);
    n = npnt / 2;
    rdf = -0.5 * dtms * npnt / tpi;
    if (verbose)
	printf("DE: npnt=%d dtms=%.3f rdf=%.3f\n", npnt, dtms, rdf);
    am = ff[0];
    ff[0] = 0;
    for (i = 1; i < n; i++) {
	ap = ff[i + 1];
	de = (ap - am) * rdf;
	am = ff[i];
	ff[i] = de;
    }
    ff[n] = 0;
}

static void
do_di()
{
    int i;
    float *d, mxmg;
    double xx, yy, xg, yg, xo, yo;
    static float x = 1.0, y = 3.0, w = 8.0, h = 4.0;

    if (!dspflg)
	return;
    if (opnout(defdev()) == 0)
	error("can't open output device for display");
    d = assign(lav[1]);
    mxmg = eps;
    for (i = 0; i < npnt; i++) {
	if (mxmg < d[i])
	    mxmg = d[i];
	if (mxmg < -d[i])
	    mxmg = -d[i];
    }
    xg = w / npnt;
    xo = x;
    yg = h / (2 * mxmg);
    yo = y + h / 2;
    for (i = 0; i < npnt; i++) {
	xx = xo + xg * i;
	yy = yo + yg * d[i];
	if (i == 0)
	    move(xx, yy);
	else
	    draw(xx, yy);
    }
    sprintf(msg, "DI: mxmg=%.3g", mxmg);
    text(1.0, 1.0, msg);
    clsout();
}

static void
do_dt()
{
    tsep = atof(lav[1]);
    dtset = 1;
    if (verbose)
	printf("tsep=%.3f\n", tsep);
}

static void
do_ft()
{
    int n;
    float *ff;

    for(n = 1; n < MXNP && n != npnt; n *= 2)	/* check for power of 2 */
	continue;
    if (n != npnt)
	error("'FT' requires that npnt be a power of 2");
    ff = assign(lav[1]);
    rfft(ff, npnt);
    ff[npnt] = ff[1];	/* unpack nyquist frequency */
    ff[1] = ff[npnt + 1] = 0;
}

static void
do_h()
{
    int i;
    static char *hlpmsg[] = {
	" AS [file]            --assigns input file",
	" CM [ar1] [ar2] [ar3] --complex multiply AR3=AR1*AR2",
	" DE [ar1] [ar2]       --converts phase AR1 to delay AR2",
	" DI [ar]              --display on scope",
	" DT [tsep]            --time density, ms",
	" FT [ar]              --fast fourier transform",
	" H                    --prints out this list ",
	" IM [ar1] [ar2]       --imaginary of AR1 => AR2",
	" LG [ar1] [ar2]       --log base 10 of AR1 => AR2",
	" LO [sb] [ar]         --loads data [SB] into array [AR]",
	" MF [ar] [file]       --moves [AR] to [FILE] by freq.",
	" MT [ar] [file]       --moves [AR] to [FILE] by time",
	" MV [ar1] [ar2]       --vector move  AR2 = AR1",
	" PD [nppo]            --point density # of points/octave",
	" Q                    --quit",
	" RC [ar1] [ar2]       --complex reciprocal AR2=(1/AR1)",
	" RE [ar1] [ar2]       --real of AR1 => AR2",
	" RP [ar1] [ar2]       --rectangular [AR1] to polar [AR2]",
	" SA [ar1] [sc] [ar2]  --scaler add AR2=(AR1+SC)",
	" SC [ar1] 	       --scale AR1 to max = 1 or min = -1",
	" SI [ar] [sc] [nc]    --sine into [AR], ampl=[SC], cyc=[NC]",
	" SM [ar1] [sc] [ar2]  --scaler mult. AR2=(AR1*SC)",
	" VA [ar1] [ar2]  [ar3]--vector add AR3=(AR1+AR2)",
	" VD [ar1] [ar2]    --vector differentiate  AR2=deriv(AR1)",
	" VI [ar1] [ar2]    --vector integrate  AR2=integ(AR1)",
	" VM [ar1] [ar2] [ar3] --vector multiply AR3=(AR1*AR2)",
	" VP [xmn] [ymn] [xmx] [ymx] --view port for output files",
        " VR [ar1] [ar2]       --vector rectify AR2=RECT(AR1)",
        " VS [ar1] [ar2] [ar3] --vector subtract AR3=(AR1-AR2)",
	" ZE [ar1] [nz]        --zero first nz values of AR1"
    };
    static int nhlp = sizeof(hlpmsg) / sizeof(hlpmsg[0]);

    for (i = 0; i < nhlp; i++)
	printf("%s\n", hlpmsg[i]);
}

static void
do_im()
{
    int i, j, n;
    float *ff, *gg;

    ff = assign(lav[1]);
    gg = assign(lav[2]);
    n = npnt / 2;
    for (i = 0; i < n; i++) {
	j = i * 2 + 1;
	gg[i] = ff[j];
    }
    for (i = n; i < npnt; i++) {
	gg[i] = 0;
    }
}

static void
do_lg()
{
    int i;
    float *ff, *gg;

    ff = assign(lav[1]);
    gg = assign(lav[2]);
    for (i = 0; i < npnt; i++) {
	gg[i] = (ff[i] > eps) ? log10(ff[i]) : -40;
    }
}

static void
do_lo()
{
    int istp;
    long hsiz, nbyt;
    float *ff;

    if (!filopn)
	error("'LO' requires file to be assigned");
    istp = atoi(lav[1]);
    if (istp < 1 || istp > nstp)
	error("'LO' subscript out of range");
    if (verbose)
	printf("LO: %d %s\n", istp, lav[2]);
    ff = assign(lav[2]);
    hsiz = pa_.hdrsiz;
    nbyt = npnt * sizeof(float);
    fseek(dfp, hsiz + nbyt * (istp - 1), 0);
    if (fread((char *) ff, nbyt, 1, dfp) < 1)
	error("reading data");
    if (swap_bytes)
        swabw((long *) ff, npnt);
}

static void
do_mf()			/* write data vs frequecy */
{
    char *ofn;
    int i, n;
    float freq, dfreq, lastf, *ff;
    FILE *ofp;

    ff = assign(lav[1]);
    strlower(ofn = lav[2]);
    if ((ofp = fopen(ofn, "w")) == NULL) {
	sprintf(msg, "can't open %s\n", ofn);
	error(msg);
    }
    if (verbose)
	printf("writing frequency data to %s\n", ofn);
    n = npnt / 2;
    lastf = 0;
    dfreq = 1 / (dtms * npnt);
    for(i = 0; i < n; i++) {
	freq = i * dfreq;
	if (vpset) {
	    if (freq < xmin || freq > xmax)
		continue;
	    if (ff[i] < ymin || ff[i] > ymax)
		continue;
	}
	if (pdset && freq > 0 && lastf > 0) {
	    if ((freq / lastf) < fsep)
		continue;
	}
	fprintf(ofp, "%11.6f %11.6f\n", freq, ff[i]);
	lastf = freq;
    }
    fclose(ofp);
}

static void
do_mp()			/* write data vs place */
{
    char *ofn;
    int i, n;
    float x, dx, lastf, *ff;
    FILE *ofp;

    ff = assign(lav[1]);
    strlower(ofn = lav[2]);
    if ((ofp = fopen(ofn, "w")) == NULL) {
	sprintf(msg, "can't open %s\n", ofn);
	error(msg);
    }
    if (verbose)
	printf("writing place data to %s\n", ofn);
    n = npnt;
    lastf = 0;
    dx = 100.0 / npnt;
    for(i = 0; i < n; i++) {
	x = i * dx;
	if (vpset) {
	    if (x < xmin || x > xmax)
		continue;
	    if (ff[i] < ymin || ff[i] > ymax)
		continue;
	}
	fprintf(ofp, "%11.6f %11.6f\n", x, ff[i]);
    }
    fclose(ofp);
}

static void
do_mt()			/* write data vs time */
{
    char *ofn;
    int i;
    float time, dtime, lastt, *ff;
    FILE *ofp;

    ff = assign(lav[1]);
    strlower(ofn = lav[2]);
    if ((ofp = fopen(ofn, "w")) == NULL) {
	sprintf(msg, "can't open %s\n", ofn);
	error(msg);
    }
    if (verbose)
	printf("writing time data to %s\n", ofn);
    lastt = 0;
    dtime = dtms;
    for(i = 0; i < npnt; i++) {
	time = i * dtime;
	if (vpset) {
	    if (time < xmin || time > xmax)
		continue;
	    if (ff[i] < ymin || ff[i] > ymax)
		continue;
	}
	if (dtset) {
	    if ((time - lastt) < tsep)
		continue;
	}
	fprintf(ofp, "%11.6f %11.6f\n", time, ff[i]);
	lastt = time;
    }
    fclose(ofp);
}

static void
do_mv()
{
    int i;
    float *ff, *gg;

    ff = assign(lav[1]);
    gg = assign(lav[2]);
    for (i = 0; i < npnt; i++) {
	gg[i] = ff[i];
    }
}

static void
do_pd()
{
    int nppo;

    nppo = atoi(lav[1]);
    if(nppo < 1)
	error("'PD' requires nppo greater than 0");
    fsep = pow(2.0, 1.0 / nppo);
    pdset = 1;
    if (verbose)
	printf("fsep=%.3f\n", fsep);
}

static void
do_rc()
{
    int i, j;
    float *ff, *gg, dn;

    ff = assign(lav[1]);
    gg = assign(lav[2]);
    for (i = 0; i < npnt; i += 2) {
	j = i + 1;
        dn = ff[i] * ff[i] + ff[j] * ff[j];
	if (dn < eps)
	    dn = eps;
	gg[i] = ff[i] / dn;
	gg[j] = -ff[j]  / dn;
    }
}

static void
do_re()
{
    int i, j, n;
    float *ff, *gg;

    ff = assign(lav[1]);
    gg = assign(lav[2]);
    n = npnt / 2;
    for (i = 0; i < n; i++) {
	j = i * 2;
	gg[i] = ff[j];
    }
    for (i = n; i < npnt; i++) {
	gg[i] = 0;
    }
}

static void
do_rp()
{
    int i, n;
    float *ff, *gg;

    ff = assign(lav[1]);
    gg = assign(lav[2]);
    for (i = 0; i < npnt; i++) {
	gg[i] = ff[i];
    }
    n = npnt / 2;
    rtop(gg, n);
    uwph(phrf_rp, gg, n, 2);
    phrf_rp=gg[1];
}

static void
do_uw()
{
    int i;
    float *ff, *gg;

    ff = assign(lav[1]);
    gg = assign(lav[2]);
    for (i = 0; i < npnt; i++) {
	gg[i] = ff[i];
    }
    uwph(phrf_uw, gg, npnt, 1);
    phrf_uw=gg[1];
}

static void
do_sa()
{
    int i;
    float *ff, *gg, sc;

    ff = assign(lav[1]);
    sc = scalar(lav[2]);
    gg = assign(lav[3]);
    if (verbose)
	printf("SA: %s %.3g %s\n", lav[1], sc, lav[3]);
    for (i = 0; i < npnt; i++) {
	gg[i] = ff[i] + sc;
    }
}

static void
do_sc()
{
    int i;
    float *ff, mxmg;

    ff = assign(lav[1]);
    mxmg = eps;
    for (i = 0; i < npnt; i++) {
	if (mxmg < ff[i])
	    mxmg = ff[i];
	if (mxmg < -ff[i])
	    mxmg = -ff[i];
    }
    for (i = 0; i < npnt; i++) {
	ff[i] /= mxmg;
    }
    if (verbose)
	printf("SC: mxmg=%9.3g\n", mxmg);
}

static void
do_si()
{
    int i;
    float *ff, sc, nc, dp;

    ff = assign(lav[1]);
    sc = scalar(lav[2]);
    nc = scalar(lav[2]);
    if (verbose)
	printf("SI: %s %.3g %s\n", lav[1], sc, lav[3]);
    dp = tpi * nc / (npnt - 1);
    for (i = 0; i < npnt; i++) {
	ff[i] = sc * sin(i * dp);
    }
}

static void
do_sm()
{
    int i;
    float *ff, *gg, sc;

    ff = assign(lav[1]);
    sc = scalar(lav[2]);
    gg = assign(lav[3]);
    if (verbose)
	printf("SM: %s %.3g %s\n", lav[1], sc, lav[3]);
    for (i = 0; i < npnt; i++) {
	gg[i] = ff[i] * sc;
    }
}

static void
do_va()
{
    int i;
    float *ff, *gg, *hh;

    ff = assign(lav[1]);
    gg = assign(lav[2]);
    hh = assign(lav[3]);
    for (i = 0; i < npnt; i++) {
	hh[i] = ff[i] + gg[i];
    }
}

static void
do_vd()
{
    float *ff, *gg;

    ff = assign(lav[1]);
    gg = assign(lav[2]);
    vderiv(ff, gg, npnt);
}

static void
do_vi()
{
    float *ff, *gg;

    ff = assign(lav[1]);
    gg = assign(lav[2]);
    vinteg(ff, gg, npnt);
}

static void
do_vm()
{
    int i;
    float *ff, *gg, *hh;

    ff = assign(lav[1]);
    gg = assign(lav[2]);
    hh = assign(lav[3]);
    for (i = 0; i < npnt; i++) {
	hh[i] = ff[i] * gg[i];
    }
}

static void
do_vp()
{
    if (lac <= 4)
	error("'VP' requires 4 arguments");
    xmin = atof(lav[1]);
    ymin = atof(lav[2]);
    xmax = atof(lav[3]);
    ymax = atof(lav[4]);
    vpset = 1;
    if (verbose)
	printf("xmin=%.3f ymin=%.3f xmax=%.3f ymax=%.3f \n",
	    xmin, ymin, xmax, ymax);
}

static void
do_vr()
{
    int i;
    float *ff, *gg;

    ff = assign(lav[1]);
    gg = assign(lav[2]);
    for (i = 0; i < npnt; i++) {
	gg[i] = (ff[i] > 0) ? ff[i] : 0;
    }
}

static void
do_vs()
{
    int i;
    float *ff, *gg, *hh;

    ff = assign(lav[1]);
    gg = assign(lav[2]);
    hh = assign(lav[3]);
    for (i = 0; i < npnt; i++) {
	hh[i] = ff[i] - gg[i];
    }
}

static void
do_ze()
{
    int i, nz;
    float *ff;

    ff = assign(lav[1]);
    nz = atoi(lav[2]);
    for (i = 0; i < nz; i++) {
	ff[i] = 0;
    }
}

static void
do_cmd(FILE *ifp)
{
    char s[80], *cmd, done = 0;
    int i;

    init_stat();
    while (fgets(s, 80, ifp) != NULL && !done) {
	lac = parse_line(s, lav);
	if (dbgflg) {
	    printf("lac=%d:", lac);
	    for (i = 0; i < lac; i++)
		printf(" %s", lav[i]);
	    printf("\n");
	}
	if (lac < 1)
	    continue;
	strlower(cmd = lav[0]);
	if (*cmd == '\0' || *cmd == '!' || *cmd == '#')
	    continue;
	else if (iscmd("as")) do_as();
	else if (iscmd("cm")) do_cm();
	else if (iscmd("de")) do_de();
	else if (iscmd("di")) do_di();
	else if (iscmd("dt")) do_dt();
	else if (iscmd("ft")) do_ft();
	else if (iscmd("h")) do_h();
	else if (iscmd("im")) do_im();
	else if (iscmd("lg")) do_lg();
	else if (iscmd("lo")) do_lo();
	else if (iscmd("mf")) do_mf();
	else if (iscmd("mp")) do_mp();
	else if (iscmd("mt")) do_mt();
	else if (iscmd("mv")) do_mv();
	else if (iscmd("pd")) do_pd();
	else if (iscmd("q")) done++;
	else if (iscmd("rc")) do_rc();
	else if (iscmd("re")) do_re();
	else if (iscmd("rp")) do_rp();
	else if (iscmd("sa")) do_sa();
	else if (iscmd("sc")) do_sc();
	else if (iscmd("si")) do_si();
	else if (iscmd("sm")) do_sm();
	else if (iscmd("uw")) do_uw();
	else if (iscmd("va")) do_va();
	else if (iscmd("vd")) do_vd();
	else if (iscmd("vi")) do_vi();
	else if (iscmd("vm")) do_vm();
	else if (iscmd("vp")) do_vp();
	else if (iscmd("vr")) do_vr();
	else if (iscmd("vs")) do_vs();
	else if (iscmd("ze")) do_ze();
	else 
	    fprintf(stderr, "%s : unknown command\n", cmd);
    }
}

static void
help()
{
    do_h();
    exit(0);
}

int
main(ac, av)
char **av;
{
    char ifn[80];
    FILE *ifp;

    while (ac > 1 && av[1][0] == '-') {
	if (av[1][1] == 'd') {
	    dbgflg = 1;
	} else if (av[1][1] == 'f') {
	    opndat(av[2]);
	    ac--;
	    av++;
	} else if (av[1][1] == 'h') {
	    help();
	} else if (av[1][1] == 's') {
	    dspflg = 1;
	} else if (av[1][1] == 'v') {
	    verbose = 1;
	}
        ac--;
 	av++;
    }

    if (ac < 2) {
	printf("DATMAN\n");
	strcpy(ifn, "stdin");
	ifp = stdin;
	do_cmd(ifp);
    } else {
	while (ac >= 2) {
	    strcpy(ifn, av[1]);
	    newext(ifn, ".cmd");
	    if (verbose)
                printf("--- %s ---\n", ifn);
	    if ((ifp = fopen(ifn, "r")) == NULL) {
	        sprintf(msg, "can't open %s\n", ifn);
	        error(msg);
	    }
	    do_cmd(ifp);
            fclose(ifp);
	    av++;
	    ac--;
	}
    }

    return (0);
}
