/* mkdm4.c - frequency to place map */

#include <stdio.h>
#include <stdlib.h>
#define MAIN
#include "tdmdef.h"
#include "cparam.h"

#define NPTS 5

double  cf();
double  atof();
double  pow();
void    swab();

static char *ifn[NPTS] = {
    "dm3a5.dat",
    "dm3a4.dat",
    "dm3a3.dat",
    "dm3a2.dat",
    "dm3a1.dat",
};
static float place[NPTS] = {90, 80, 60, 40, 20};

/* swabw - swap bytes (in place) for n "words" */
void
swabw(p, n)
long   *p;
int n;
{
    unsigned char *ch, tmp;

    while (n-- > 0) {
	ch = (unsigned char *) p++;
	tmp = ch[3];
	ch[3] = ch[0];
	ch[0] = tmp;
	tmp = ch[2];
	ch[2] = ch[1];
	ch[1] = tmp;
    }
}

static void
opndat(fn)
char *fn;
{
    float oonn;
    int i;
    FILE *dfp;

    if ((dfp = fopen(fn, "rb")) == NULL) {
	fprintf(stderr, "ERROR: can't open %s\n", fn);
	exit(1);
    }
    if (fread((char *) &pa_, sizeof(pa_), 1, dfp) < 1) {
	fprintf(stderr, "ERROR: reading header\n");
	exit(1);
    }
    if (pa_.ireor > 255) {	/* check for byte swap */
	swabw((long *)&pa_.magic, 1);
	swab((void*)&pa_.hdrsiz, (void *)&pa_.hdrsiz, 6 * sizeof(short) );
	swabw((long *)&pa_.dt, 14);
	swab((void*)&pa_.m, (void*)&pa_.m, 21 * sizeof(short));
	swabw((long *)&pa_.xl, 164);
    }
    fclose(dfp);
    oonn = (pa_.n > 1) ? 1.0 / (pa_.n - 1) : 0;
    for (i = 0; i < 5; i++)
	place[i] = 100 * pa_.isv[4 - i] * oonn;
}

static void
fpmap_(ofn, a, b, c, spnm)
char *ofn, *spnm;
double a, b, c;
{
    int     i;
    float   f[101], p[101];
    FILE   *ofp;

    for (i = 0; i < 101; i++) {
	p[i] = i;
	f[i] = a * (pow(10.0, b * (100 - i)) - c);
    }
    if ((ofp = fopen(ofn, "w")) == NULL) {
	fprintf(stderr, "can't open %s\n", ofn);
	exit(0);
    }
	fprintf(ofp, 
	    ";%s - Greenwood frequency-to-place map for %s\n"
	    ";\n"
	    ";	a=%.4f,  b=%.3f, c=%.3f\n"
	    ";\n"
	    ";	x=frequency (kHz)\n"
	    ";	y=percent distance from base\n"
	    ";\n", ofn, spnm, a, b, c);
    for (i = 0; i < 101; i++)
	fprintf(ofp, "%.3f %.3f\n", f[i], p[i]);
    fclose(ofp);
}

static void
fpmap2(ofn)
char *ofn;
{
    int     i;
    float   f[NPTS], p[NPTS];
    FILE   *ofp;

    for (i = 0; i < NPTS; i++) {
	f[i] = cf(ifn[i]);
	p[i] = place[i];
    }
    if ((ofp = fopen(ofn, "w")) == NULL) {
	fprintf(stderr, "can't open %s\n", ofn);
	exit(0);
    }
	fprintf(ofp, 
	    ";%s - model frequency-to-place map\n"
	    ";\n"
	    ";	x=frequency (kHz)\n"
	    ";	y=percent distance from base\n"
	    ";\n", ofn);
    for (i = 0; i < NPTS; i++)
	fprintf(ofp, "%.3f %.3f\n", f[i], p[i]);
    fclose(ofp);
}

/* cf - characteristic frequency */

double
cf(fn)
char   *fn;
{
    char    line[80];
    int     i, j;
    float   f, p, fmin, pmin;
    FILE   *ifp;

    fmin = 0;
    pmin = 1e18;
    if ((ifp = fopen(fn, "r")) == NULL) {
	fprintf(stderr, "can't open %s\n", fn);
	return (0);
    }
    for (i = 0; fgets(line, 80, ifp) != NULL; i++) {
	if ((f = atof(line)) > 0) {
	    for (j = 0; line[j] == ' '; j++)
		continue;
	    for (; line[j] > ' '; j++)
		continue;
	    for (; line[j] == ' '; j++)
		continue;
	    p = atof(line + j);
	    if (pmin > p) {
		pmin = p;
		fmin = f;
	    }
	}
    }
    fclose(ifp);
    return (fmin);
}

int
main(int ac, char **av)
{
    opndat((ac == 2) ? av[1] : "tdm.tsr");
    fpmap_("dm4a0.dat", 0.1654, 0.021, 0.99, "man");
    fpmap_("dm4a1.dat", 0.4560, 0.021, 0.80, "cat");
    fpmap2("dm4a2.dat");

    return (0);
}
