/* rtop.c - convert complex array from rectangular to polar coordinates */

#define abs(x) 		((x < 0) ? -x : x)
#define min(x,y) 	((x < y) ? x : y)
#define max(x,y) 	((x < y) ? y : x)
#define round(x) 	((x < 0) ? ((int) (x - 0.5)) : ((int) (x + 0.5)))
#define uw(p,r)		(p + round((r - p) / tpi) * tpi)

double  sqrt();
double  atan2();

static double tpi = 6.2831854;

void
rtop(float *y, int n)
{
    int     i, j;
    float   yi, yr, yia, yra, yb, yc;
    float   mg, ph;
    static float big1 = 1e19;
    static float big2 = 1e38;

    for (i = 0; i < (n * 2); i += 2) {
	j = i + 1;
	yr = y[i];
	yi = y[j];
	yra = abs(yr);
	yia = abs(yi);
	yb = max(yra, yia);
	/* compute db and ph */
	if (yb == 0) {
	    mg = 0;
	    ph = 0;
	} else if (yb < big1) {	/* check for overflow */
	    mg = sqrt(yr * yr + yi * yi);
	    ph = atan2(yi, yr);
	} else if (yb < big2) {
	    yc = min(yra, yia) / yb;
	    mg = sqrt(1 + yc * yc) * yb;
	    ph = atan2(yi / yb, yr / yb);
	} else {
	    mg = big2;
	    ph = 0;
	}
	y[i] = mg;
	y[j] = ph;
    }
}

/* uwph - unwrap phase of complex number in polar form (minimum delay) */

void
uwph(double phrf, float *y, int n, int m)
{
    int     i;

    for (i = 1; i < (n * m); i += m) {
	y[i] = uw(y[i], phrf);
	phrf = y[i];
    }
}
