/* dspsta.c - main program for time domain model */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#define MAIN
#include "tdmdef.h"
#include "cparam.h"
#include "cstate.h"
#include "outlib.h"

void displa();
void dspvar();
void error();
void newext();
void readst();
void swab();
void tdmarg();

static char ifn[80] = "tdm.sta";
static int swap_bytes = 0;

int
main(ac, av)
char  **av;
{
    tdmarg(ac, av);	/* interpret command line args */
    readst();		/* read state variables from disk */
    displa();		/* display state */
    return(0);		/* done */
}

void
tdmarg(ac, av)
char  **av;
{
    while (ac > 1) {
	if (av[1][0] == '-') {
	    switch (av[1][1]) {
	    }
	} else {
	    strcpy(ifn, av[1]);
	    newext(ifn, ".sta");
	}
	ac--;
	av++;
    }
}

/* swabw - swap bytes (in place) for n "words" */
void
swabw(p, n)
long   *p;
int n;
{
    unsigned char *ch, tmp;

    while (n-- > 0) {
	ch = (unsigned char *) p++;
	tmp = ch[3];
	ch[3] = ch[0];
	ch[0] = tmp;
	tmp = ch[2];
	ch[2] = ch[1];
	ch[1] = tmp;
    }
}

void
readst()
{
    FILE *sfp;

    sfp = fopen(ifn, "rb");

    /* read parameter block */
    fseek(sfp, 0L, 0);
    if (fread((char *) &pa_, sizeof(pa_), 1, sfp) < 1)
	error("error reading state (header)");
    if (pa_.ireor > 255) {	/* check for byte swap */
	swabw((long *)&pa_.magic, 1);
	swab((void*)&pa_.hdrsiz, (void *)&pa_.hdrsiz, 6 * sizeof(short) );

	swab((void*)&pa_.m, (void*)&pa_.m, 21 * sizeof(short));
	swabw((long *)&pa_.xl, 164);
	swap_bytes++;
    }
fprintf(stderr, "opened %s, swap_bytes=%d\n", ifn, swap_bytes);
    /* read state */
    fseek(sfp, pa_.hdrsiz, 0);
    if (fread((char *) &st_, sizeof(st_), 1, sfp) < 1)
	error("error reading state");
    if (swap_bytes)
        swabw((long *) &st_, sizeof(st_) / sizeof(float));
    fclose(sfp);
}

void
displa()
{
    if (opnout(defdev()) == 0)
	error("can't open graphics device");
    setfais(1);
    setfasi(0);
    text(0.5, 6.5, "d1");
    dspvar(1.0, 6.0, 8.0, 1.0, pa_.n, st_.d_ + ST_OFF_1);
    text(0.5, 4.5, "d2");
    dspvar(1.0, 4.0, 8.0, 1.0, pa_.n, st_.d_ + ST_OFF_2);
    clsout();
}

void
dspvar(x, y, w, h, n, d)
float x, y, w, h, *d;
{
    int i, imxmg;
    double dmg, dmxmg;
    double xx, yy, xg, yg, xo, yo;

    dmxmg = 1e-38;
    imxmg = -1;
    for (i = 0; i < n; i++) {
	dmg = (d[i] < 0) ? -d[i] : d[i];
	if (dmxmg < dmg) {
	    dmxmg = dmg;
	    imxmg = i;
	}
    }
    if (imxmg < 0) {
	move(x, y + h / 2);
	draw(x + w, y + h / 2);
	return;
    }

    xg = w / n;
    xo = x;
    yg = h / (2 * dmxmg);
    yo = y + h / 2;
    for (i = 0; i < n; i++) {
	xx = xo + xg * i;
	yy = yo + yg * d[i];
	if (i == 0)
	    move(xx, yy);
	else
	    draw(xx, yy);
    }
}

void
newext(f, e)
char   *f, *e;
{
    char   *s;

    s = f;
    while (*s != '\0')		/* locate end of filename */
	s++;
    while (*s != '\\' && *s != '/' && s > f)	/* backup to start of name */
	s--;
    while (*s != '\0' && *s != '.')	/* locate end of name */
	s++;
    while (*e != '\0')		/* copy extension */
	*s++ = *e++;
    *s = '\0';
}

void
error(s)
char *s;
{
    fprintf(stderr, "*** Error: %s\n", s);
    exit(1);
}
