/* reor.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "tdmdef.h"
#define MAIN
#include "cparam.h"

void error();
void newext();
void swab();

static char fn1[40] = "tdm.tsd";
static char fn2[40] = "tdm.tsr";
static int dbgflg = 1;
static int swap_bytes = 0;
static FILE *fp1, *fp2;

/* swabw - swap bytes (in place) for n "words" */
void
swabw(p, n)
long   *p;
int n;
{
    unsigned char *ch, tmp;

    while (n-- > 0) {
	ch = (unsigned char *) p++;
	tmp = ch[3];
	ch[3] = ch[0];
	ch[0] = tmp;
	tmp = ch[2];
	ch[2] = ch[1];
	ch[1] = tmp;
    }
}

int
main(ac, av)
char  **av;
{
    char    msg[80];
    int     nrows, ncols, npts, i, j;
    long    hsiz;
    float  *a, *b;


    if (ac > 2) {
	strcpy(fn1, av[1]);
	newext(fn1, ".tsd");
	strcpy(fn2, av[2]);
	newext(fn2, ".tsr");
    } else if (ac > 1) {
	strcpy(fn1, av[1]);
	newext(fn1, ".tsd");
	strcpy(fn2, av[1]);
	newext(fn2, ".tsr");
    }

    if ((fp1 = fopen(fn1, "rb")) == NULL) {
	sprintf(msg, "can't open %s\n", fn1);
	error(msg);
    }
    fprintf(stderr, "opened %s\n", fn1);
    if (fread((char *) &pa_, sizeof(pa_), 1, fp1) < 1) {
	error("reading header\n");
    }
    if (pa_.ireor > 255) {	/* check for byte swap */
	swabw((long *)&pa_.magic, 1);
	swab((void*)&pa_.hdrsiz, (void *)&pa_.hdrsiz, 6 * sizeof(short) );
	swabw((long *)&pa_.dt, 14);
	swab((void*)&pa_.m, (void*)&pa_.m, 21 * sizeof(short));
	swabw((long *)&pa_.xl, 164);
	swap_bytes++;
    }
    if (dbgflg) {
	printf(" ireor,ifrmt,jfrmt    = %4d%4d%4d\n",
	    pa_.ireor, pa_.ifrmt, pa_.jfrmt);
	printf(" itsc,icyc,itsw,itsf  = %8ld%8ld%8ld%8ld\n",
	    pa_.itsc, pa_.icyc, pa_.itsw, pa_.itsf);
	printf(" ntsc,ncyc,ntsw,ntsf  = %8ld%8ld%8ld%8ld\n",
	    pa_.ntsc, pa_.ncyc, pa_.ntsw, pa_.ntsf);
    }
    if (pa_.ireor != 0) {
	sprintf(msg, "wrong type of data file: ireor=%d\n", pa_.ireor);
	error(msg);
    }
    ncols = pa_.ntsf;
    switch (pa_.ifrmt) {
    case 1:
	nrows = pa_.n;
	break;
    case 2:
	nrows = pa_.n * 2;
	break;
    case 3:
    case 7:
    case 8:
	nrows = 16;
	break;
    case 5:
    case 6:
	nrows = 256;
	break;
    default:
	sprintf(msg, "format not implemented: ifrmt=%d\n", pa_.ifrmt);
	error(msg);
    }

    npts = nrows * ncols;
    printf("nrows=%d  ncols=%d  npts=%d\n", nrows, ncols, npts);

    a = (float *) malloc(nrows * sizeof(float));
    b = (float *) malloc(nrows * ncols * sizeof(float));
    if (a == NULL || b == NULL) {
	error("allocating memory\n");
    }
    hsiz = pa_.hdrsiz;
    fseek(fp1, hsiz, 0);
    for (i = 0; i < ncols; i++) {
	if (fread((char *) a, sizeof(float), nrows, fp1) < nrows) {
	    /* error("reading data\n") */;
	}
	if (swap_bytes)
	    swabw((long *) a, nrows);
	for (j = 0; j < nrows; j++)
	    b[i + j * ncols] = a[j];
    }
    fclose(fp1);

    pa_.ireor = 1;		/* identify output file as reordered data */
    if ((fp2 = fopen(fn2, "wb")) == NULL) {
	sprintf(msg, "can't open %s\n", fn2);
	error(msg);
    }
    if (fwrite((char *) &pa_, sizeof(pa_), 1, fp2) < 1) {
	error("writing header\n");
    }
    fseek(fp2, hsiz, 0);
    if (fwrite((char *) b, sizeof(float) * nrows, ncols, fp2) < ncols) {
	error("writing data\n");
    }
    fclose(fp2);

    free(a);
    free(b);
    return(0);
}

void
newext(f, e)
char   *f, *e;
{
    char   *s;

    s = f;
    while (*s != '\0')		/* locate end of filename */
	s++;
    while (*s != '\\' && *s != '/' && s > f)	/* backup to start of name */
	s--;
    while (*s != '\0' && *s != '.')	/* locate end of name */
	s++;
    while (*e != '\0')		/* copy extension */
	*s++ = *e++;
    *s = '\0';
}

void
error(s)
char   *s;
{
    fprintf(stderr, "ERROR: %s\n", s);
    exit(1);
}
