/* tdm_xw.c */

#ifdef XW

#include <stdlib.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include "tdmdef.h"

static Colormap cmap;
static Display *mdsp;
static Window  mwin;
static GC      mgc;
static GC      egc;
static unsigned long wp;
static unsigned long bp;

void
xw_init(int *xs, int *ys)
{
    int     scrn;
    int     dpth;
    XSetWindowAttributes swa;
    XWindowAttributes wa;
    unsigned long wmsk;
    XSizeHints theSizeHints;
    XGCValues gcv;

    int     x = 8;		/* x top left corner of window */
    int     y = 8;		/* y top left corner of window */
    unsigned int width = 640;	/* width of the window */
    unsigned int height = 480;	/* height of the window */
    int     border_width = 2;	/* border width of the window */

    mdsp = XOpenDisplay("");
    if (mdsp == NULL) {
	return;
    }
    scrn = DefaultScreen(mdsp); 
    dpth = DefaultDepth(mdsp, scrn); 
    wp = WhitePixel(mdsp, scrn);
    bp = BlackPixel(mdsp, scrn);
    cmap = DefaultColormap(mdsp, scrn); 
    x = (XDisplayWidth(mdsp, scrn) - width) / 2;
    y = 8;
    swa.border_pixel = BlackPixel(mdsp, scrn); 
    swa.background_pixel = WhitePixel(mdsp, scrn); 
    swa.override_redirect = True;
    swa.backing_store = WhenMapped;
    wmsk = CWBackPixel | CWBorderPixel | CWOverrideRedirect | CWBackingStore; 
    mwin = XCreateWindow(mdsp,
	RootWindow(mdsp, scrn),
	x, y, width, height, border_width,
	dpth, InputOutput, CopyFromParent,
	wmsk, &swa);

    theSizeHints.flags = PPosition | PSize;	/* set mask for the hints */
    theSizeHints.x = x;		/* x position */
    theSizeHints.y = y;		/* y position */
    theSizeHints.width = width;	/* width of the window */
    theSizeHints.height = height;	/* height of the window */

    XSetNormalHints(mdsp, mwin, &theSizeHints);

    mgc = XCreateGC(mdsp, mwin, (unsigned long) 0, &gcv);
    egc = XCreateGC(mdsp, mwin, (unsigned long) 0, &gcv);

    /* error... cannot create gc */
    if (mgc == 0 || egc == 0) {
	XDestroyWindow(mdsp, scrn);
	exit(0);
    }
    /* set forground and background defaults */
    else {
	XSetForeground(mdsp, mgc, bp);
	XSetBackground(mdsp, mgc, wp);
	XSetForeground(mdsp, egc, wp);
	XSetBackground(mdsp, egc, wp);
    }

    XMapRaised(mdsp, mwin);
    XGetWindowAttributes(mdsp, mwin, &wa);

    *xs = wa.width;
    *ys = wa.height;
}

void
xw_quit()
{
    XDestroyWindow(mdsp, mwin);
    XCloseDisplay(mdsp);
}

void
xw_line(int x1, int y1, int x2, int y2, int y3, int y4)
{
    XDrawLine(mdsp, mwin, egc, x1, y1, x2, y2);
    XDrawLine(mdsp, mwin, mgc, x1, y3, x2, y4);
}

#endif /* XW */

