/* tlook.c  - display and plot tdm data files */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "tdmdef.h"
#define MAIN
#include "cparam.h"
#include "cstate.h"
#include "outlib.h"

#define MAXSAV          (2 * (NDOF + 1))
#define FRAMES_PER_SEC  20
#define limit(a,b,c)	(((a)>(b))?(a):((c)<(b))?(c):(b))

void dspvar();
void error();
void prnmax();
void prntim();
void prnvar();
void swab();
void t_axis();
void timstp();
void usage();

static int dbgflg = 0;
static int movflg = 0;
static int wriflg = 0;
static int prsflg = 0;
static int maxflg = 0;
static int fm8flg = 0;
static int swap_bytes = 0;
static char *tsrtxt[16] = {
    "vep",
    "cen",
    "wnr",
    "dsav[pa_.isv[4]]",
    "Ped",
    "Dma",
    "VVed",
    "Ped * VVed",
    "Pst",
    "Dst",
    "VVst",
    "Pst * VVst",
    "dsav[pa_.isv[0]]",
    "dsav[pa_.isv[1]]",
    "dsav[pa_.isv[2]]",
    "dsav[pa_.isv[3]]"
};

/* sswab - swap bytes (in place) for n "short ints" */
void
sswab(void *p, int n)
{
    swab(p, p, n * 2); 
}

/* lswab - swap bytes (in place) for n "long ints" */
void
lswab(long *p, int n)
{
    unsigned char *ch, tmp;

    while (n-- > 0) {
	ch = (unsigned char *) p++;
	tmp = ch[3];
	ch[3] = ch[0];
	ch[0] = tmp;
	tmp = ch[2];
	ch[2] = ch[1];
	ch[1] = tmp;
    }
}

/* hdr_swab - swap header bytes  */
void
hdr_swab()
{
    sswab((void *) &pa_.hdrsiz, 6);
    lswab((long *) &pa_.dt, 14);
    sswab((void *) &pa_.m, 21);
    lswab((long *) &pa_.xl, 168);
}

int
main(ac, av)
char  **av;
{
    char   *fn1, msg[80];
    int     istp, lstp;
    FILE   *fp1;

    while (ac > 1 && av[1][0] == '-') {
	if (av[1][1] == 'd')
	    dbgflg = 1;
	else if (av[1][1] == 'm')
	    movflg = 1;
	else if (av[1][1] == 'p')
	    prsflg = 1;
	else if (av[1][1] == 'w')
	    wriflg = 1;
	else if (av[1][1] == 'x')
	    maxflg = 1;
	ac--;
	av++;
    }

    if (ac < 2)
	usage();
    fn1 = av[1];
    istp = (ac > 2) ? atoi(av[2]) : 0;
    lstp = (ac > 3) ? atoi(av[3]) : 0;

    if ((fp1 = fopen(fn1, "rb")) == NULL) {
	sprintf(msg, "can't open %s\n", fn1);
	error(msg);
    }
    if (fread((char *) &pa_, sizeof(pa_), 1, fp1) < 1) {
	error("reading header\n");
    }
    if (pa_.ifrmt > 255) {	/* check for byte swap */
	hdr_swab();
	swap_bytes++;
    }
    if (dbgflg) {
	printf(" ireor,ifrmt,jfrmt    = %4d%4d%4d\n",
	    pa_.ireor, pa_.ifrmt, pa_.jfrmt);
	printf(" itsc,icyc,itsw,itsf  = %8ld%8ld%8ld%8ld\n",
	    pa_.itsc, pa_.icyc, pa_.itsw, pa_.itsf);
	printf(" ntsc,ncyc,ntsw,ntsf  = %8ld%8ld%8ld%8ld\n",
	    pa_.ntsc, pa_.ncyc, pa_.ntsw, pa_.ntsf);
    }
    if (pa_.ireor < -1 || pa_.ireor > 2) {
	sprintf(msg, "undefined file type: ireor=%d\n", pa_.ireor);
	error(msg);
    }
    timstp(fp1, istp, lstp);
    fclose(fp1);
    return(0);			/* done */
}

void
timstp(fp, istp, lstp)
FILE   *fp;
{
    char   *ftyp, msg[80];
    double  delt;
    int     stp, incr;
    int     nbyt, nwrd;
    int     npts_st[MAXSAV], i, j, k, o, n, ipnt = 0;
    long    ofst_st[MAXSAV];
    long    ofst, nstp, npts, next_clock = 0;
    float  *abuf;
    char   *dtyp_st[MAXSAV] = {"dme", "d1", "d2", "vme", "v1", "v2"};

    /* determine nblk, nstp, and npts */
    if (pa_.ireor == -1) {	/* is it a state file ? */
	ftyp = "state";
	nbyt = Nmax * sizeof(float);
	npts = pa_.n;
	nstp = MAXSAV;
        for (j = 0; j < 2; j++) {
            k = j * (NDOF + 1);
	    npts_st[k] = n = pa_.nmev;
	    ofst_st[k] = o = pa_.hdrsiz + j * NSTVmax * sizeof(float);
	    for (i = 1; i <= NDOF; i++) {
		npts_st[k + i] = npts;
		ofst_st[k + i] = o + (n + i * npts) * sizeof(float);
	    }
	}
    } else if (pa_.ireor == 0) {/* is it a time step data file ? */
	ftyp = "time-step data";
	nstp = pa_.ntsf;
	switch (pa_.ifrmt) {
	case 1:
	    nbyt = pa_.n * sizeof(float);
	    npts = pa_.n;
	    ipnt = 1;
	    break;
	case 2:
	    nbyt = pa_.n * sizeof(float);
	    npts = pa_.n;
	    ipnt = 1;
	    nstp = pa_.ntsf;
	    break;
	case 4:
	case 5:
	case 6:
	    nbyt = 256 * sizeof(float);
	    npts = 120;
	    break;
	case 3:
	case 7:
	case 8:
	    nbyt = 16 * sizeof(float);
	    npts = 16;
	    break;
	default:
	    sprintf(msg, "*** Undefined format: ifmrt=%d\n", pa_.ifrmt);
	    error(msg);
	}
    } else if (pa_.ireor == 1) {
	ftyp = "reordered data";
	npts = pa_.ntsf;
	nbyt = pa_.ntsf * sizeof(float);
	delt = (pa_.dt * pa_.ntsw) * pa_.ntsf;
	printf("delt=%.3e\n", delt);
	switch (pa_.ifrmt) {
	case 1:
	    nstp = pa_.n;
	    break;
	case 2:
	    nstp = pa_.n;
	    break;
	case 4:
	case 5:
	case 6:
	    nstp = 256;
	    break;
	case 3:
	case 7:
	case 8:
	    nstp = 16;
	    fm8flg = 1;
	    break;
	default:
	    sprintf(msg, "*** Undefined format: ifmrt=%d\n", pa_.ifrmt);
	    error(msg);
	}
    } else if (pa_.ireor == 2) {/* is it a FT data file ? */
	ftyp = "Fourier transformed data";
	nstp = pa_.ntsf;
	switch (pa_.ifrmt) {
	case 1:
	    nbyt = pa_.n * sizeof(float);
	    npts = pa_.n;
	    ipnt = 1;
	    break;
	case 2:
	    nbyt = pa_.n * sizeof(float);
	    npts = pa_.n;
	    ipnt = 1;
	    nstp = pa_.ntsf;
	    break;
	case 4:
	case 5:
	case 6:
	    nbyt = 256 * sizeof(float);
	    npts = 120;
	    break;
	case 3:
	case 7:
	case 8:
	    nbyt = 16 * sizeof(float);
	    npts = 16;
	    break;
	default:
	    sprintf(msg, "*** Undefined format: ifmrt=%d\n", pa_.ifrmt);
	    error(msg);
	}
    }
    printf("%s file:  ", ftyp);
    printf("nstp=%ld, npts=%ld, nbyt=%d\n", nstp, npts, nbyt);
    printf("itsf=%ld, icyc=%ld, ", pa_.itsf, pa_.icyc);
    printf("ntsf=%ld, ncyc=%ld\n", pa_.ntsf, pa_.ncyc);
    lstp = (lstp > 0) ? lstp : (istp > 0) ? istp : nstp;
    istp = (istp > 0) ? istp : 1;
    incr = 1;
    if (fm8flg && istp == lstp)
	printf("%s\n", tsrtxt[istp - 1]);
    else
	printf("istp=%d, lstp=%d\n", istp, lstp);

    abuf = (float *) malloc(nbyt);
    if (abuf == NULL)
	error("can't allocate memory");

    if (wriflg == 0 && maxflg == 0) {
	printf("press return to continue ... \n");
	getchar();
	if (opnout(defdev())== 0)
	    error("can't open graphics device");
	setchh(0.15);
    }
    if (movflg) {
	setpgp(0);
	setpgb(0);
	next_clock = clock();
    }
    do {
	for (stp = istp; stp <= lstp; stp += incr) {

            if (movflg) {
		while(clock() < next_clock)
		    continue;
	        next_clock = clock() + CLOCKS_PER_SEC / FRAMES_PER_SEC;
	    }

	    /* read data */
	    if (pa_.ireor == -1) {
        	npts = npts_st[stp - 1];
		nbyt = npts * sizeof(float);
		ofst = ofst_st[stp - 1];
            } else if (pa_.ifrmt == 2) {
	        if (pa_.ireor == 0) {
 		    ofst = pa_.hdrsiz + nbyt * (stp - 1) * 2;
	            ofst += prsflg ? 0 : nbyt;
	        } else {
 		    ofst = pa_.hdrsiz + nbyt * (stp - 1);
	            ofst += prsflg ? 0 : nbyt * nstp;
	        }
	    } else {
		ofst = pa_.hdrsiz + nbyt * (stp - 1);
	    }
	    if (dbgflg)
		printf("(1) stp=%d\n", stp);
	    fseek(fp, ofst, 0);
	    nwrd = nbyt / sizeof(float);
	    if (fread((char *) abuf, nwrd, sizeof(float), fp) < 1) {
		if (wriflg == 0 && maxflg == 0)
		    clsout();
		error("reading data");
	    }
	    if (swap_bytes)
	    	lswab((long *) abuf, nwrd);
	    if (dbgflg)
		printf("(2) stp=%d\n", stp);
	    if (wriflg) {
		prnvar(stp, npts, abuf);
	    } else if (maxflg) {
		prnmax(ipnt, npts, abuf, stp);
	    } else {
		if (stp > istp || movflg)
		    page();
		if (movflg) 
		    prntim(stp * pa_.dt * pa_.ntsw);
		dspvar(1.0, 3.0, 8.0, 4.0, ipnt, npts, abuf, stp);
		if (pa_.ireor == 1)
		    t_axis(1.0, 3.0, 8.0, 0.2, delt);
		if (fm8flg)
		    text(1.0, 1.1, tsrtxt[stp - 1]);
		else if (pa_.ireor == -1)
		    text(1.0, 1.1, dtyp_st[stp - 1]);
		else if (pa_.ifrmt == 2)
		    text(1.0, 1.1, prsflg ? "pres." : "vel.");
		if (movflg)
		    dspvar(1.0, 3.0, 8.0, 4.0, ipnt, npts, abuf, stp);
	    }
	    if (dbgflg)
		printf("(3) stp=%d\n", stp);
	}
    } while (movflg);
    if (wriflg == 0 || maxflg) {
	if (movflg) {
	    setpgp(1);
	    setpgb(1);
	}
	clsout();
    }
    free(abuf);
}

void
dspvar(x, y, w, h, m, n, d, stp)
float   x, y, w, h, *d;
{
    char    s[80];
    int     i, imx;
    double  dmg, dmx;
    double  xx, yy, xg, yg, xo, yo;
    static double dmxmov = 0;

    dmx = 1e-38;
    imx = -1;
    for (i = 0; i < m; i++)
	d[i] = 0;
    for (i = m; i < n; i++) {
	if (d[i] != d[i])	/* test for NaN */
	    d[i] = 0;
	dmg = (d[i] < 0) ? -d[i] : d[i];
	if (dmx < dmg) {
	    dmx = dmg;
	    imx = i;
	}
    }
    if (movflg) {
	if (dmxmov < dmx)
	    dmxmov = dmx;
	dmx = dmxmov;
    }
    if (imx < 0) {
	move(x, y + h / 2);
	draw(x + w, y + h / 2);
	sprintf(s, "stp=%d, no response", stp);
    } else {
	xg = w / n;
	xo = x;
	yg = h / (2 * dmx);
	yo = y + h / 2;
	move(x, y + h / 2);
	for (i = 0; i < n; i++) {
	    xx = xo + xg * i;
	    yy = yo + yg * d[i];
	    yy = limit(y, yy, y + h);
	    draw(xx, yy);
	}
	sprintf(s, "stp=%d, dmx=%9.3e at %d/%d", stp, dmx, imx, n);
        if (pa_.ireor == 2)
            sprintf(s, "=%.0f", ((stp - 1) / 2) * 38.75);
    }
    if (!movflg)
	text(1.0, 0.8, s);
}

void
prnmax(m, n, d, stp)
float  *d;
{
    int     i, imx;
    double  dmg, dmx;

    dmx = 1e-38;
    imx = -1;
    for (i = 0; i < m; i++)
	d[i] = 0;
    for (i = m; i < n; i++) {
	if (d[i] != d[i])	/* test for NaN */
	    d[i] = 0;
	dmg = (d[i] < 0) ? -d[i] : d[i];
	if (dmx < dmg) {
	    dmx = dmg;
	    imx = i;
	}
    }
    printf("stp=%d, dmx=%9.3e at %d/%d\n", stp, dmx, imx, n);
}

void
prnvar(stp, n, d)
float  *d;
{
    char    fn[80];
    double  dx;
    int     i;
    FILE   *fp;

    dx = 1.0 / n;
    sprintf(fn, "step.%d", stp);
    fp = fopen(fn, "w");
    for (i = 0; i < n; i++)
	fprintf(fp, "%6.3f %12.3e\n", i * dx, d[i]);
    fclose(fp);
}

void
prntim(double t)
{
    char    s[20];

    sprintf(s, "time=%5.2f ms  ", t * 1000);
    text(1.0, 2.5, s);
}

void
t_axis(x, y, w, h, dt)
float   x, y, w, h, dt;
{
    double  dx, xx, dy;

    move(x, y);
    draw(x + w, y);
    dx = 0.01 * w / dt;
    dy = h;
    for (dx = 0.01 * w / dt; dx > 0.1; dx *= 0.1, dy *= 0.5) {
	for (xx = x; xx <= x + w; xx += dx) {
	    move(xx, y);
	    draw(xx, y - dy);
	}
    }
}

void
error(char *s)
{
    fprintf(stderr, "ERROR:  %s\n", s);
    exit(1);
}

void
usage()
{
    fprintf(stderr, "\n");
    fprintf(stderr, "usage:  tlook [-options] tdm_data_file [time_step]\n");
    fprintf(stderr, "\n");
    fprintf(stderr, "options:\n");
    fprintf(stderr, "        -m  show movie\n");
    fprintf(stderr, "        -p  show pressure when ifrmt=2\n");
    fprintf(stderr, "        -w  write data values\n");
    fprintf(stderr, "        -x  print maximum value\n");
    exit(1);
}
